// This work is licensed under the Creative Commons Attribution-ShareAlike 4.0
// International Public License.
// Visit https://creativecommons.org/licenses/by-sa/4.0/.
// SPDX-License-Identifier: CC-BY-SA-4.0
//     yuqk pattern_modifiers_00.pov +W600 +h450 (faster; uglier)
//     yuqk pattern_modifiers_00.pov +W1200 +h900 +a0.05 +am2 +r3 +j7.33 +d +p
#version unofficial 3.8; // yuqk
#if (file_exists("version.inc"))
    #include "version.inc"
#end
#if (!defined(Fork_yuqk))
    #error "This POV-Ray SDL code requires the yuqk fork."
#end
global_settings { assumed_gamma 1 }
#declare Grey50 = srgb <0.5,0.5,0.5>;
background { Grey50 }
#declare Camera00 = camera {
    perspective
    location <3,3,-3.001>
    sky y
    angle 35
    right x*(image_width/image_height)
    look_at <0,0,0>
}
#declare White = srgb <1,1,1>;
#declare Light00 = light_source { <50,150,-250>, White }
// The focus is just below. It shows the use of the pattern_modifiers keyword 
// as a way to use the spacial modifiers POV-Ray users know to manipulate inbuilt 
// functions. Below the '00' suffixed set is banging on he f_sphere() inbuilt 
// function. The second with '01..03' suffixes twice uses f_helix1() inbuilt. 
#include "functions.inc"
// Function f_sphere defined in functions.inc
// Function f_helix1 defined in functions.inc
// Function f_min1to8pairs defined in functions.inc
// Bang on a f_sphere() function
#declare Black = srgb <0,0,0>;
#declare PigMods00 = pigment {
    Black
    rotate 45*x
    scale <1.3,2.3,0.1>
    rotate -15*y
    scale 9
    warp { turbulence <0.1,0.1,0.1> octaves 5 omega 0.07 lambda 7.75 }
    scale 1/9
    rotate x*60 rotate z*35
    translate <0.1,0.1,-0.1>
    scale 1.4
}
#declare FnPigMods00 = function { pattern_modifiers { PigMods00 } }
#declare Fn00 = function (x,y,z) {
    f_sphere(FnPigMods00(x,y,z).x,FnPigMods00(x,y,z).y,FnPigMods00(x,y,z).z,0.3)
}
// Set up two functions using f_helix1()
#declare PigMods01 = pigment {
    Black
    rotate 45*x
    scale <1.0,1.0,1>
    translate 0.2
    warp { turbulence <0.3,0.3,0.3> octaves 4 omega 0.09 lambda 5.75 }
    translate -0.2
    rotate x*20 rotate z*5
    translate <0.1,0.1,-0.1>
}
#declare FnPigMods01 = function { pattern_modifiers { PigMods01 } }
#declare Fn01 = function (x,y,z) {
    f_helix1(FnPigMods01(x,y,z).x,FnPigMods01(x,y,z).y,FnPigMods01(x,y,z).z,
    9,-0.65,0.010,0.4,30,1,45)
}
#declare PigMods02 = pigment {
    Black
    rotate (360/18)*y rotate 45*x
    scale <1.0,1.0,1>
    translate 0.2
    warp { turbulence <0.3,0.3,0.3> octaves 4 omega 0.09 lambda 5.75 }
    translate -0.2
    rotate x*20 rotate z*5
    translate <0.1,0.1,-0.1>
}
#declare FnPigMods02 = function { pattern_modifiers { PigMods02 } }
#declare Fn02 = function (x,y,z) {
    f_helix1(FnPigMods02(x,y,z).x,FnPigMods02(x,y,z).y,FnPigMods02(x,y,z).z,
    9,-0.65,0.010,0.4,30,1,45)
}
// Set up a merge of the two using f_helix1(). Using inbuilt function
// f_min1to8pairs().
#declare FnHuge = function (x,y,z) { 999 }
#declare Fn03 = function (x,y,z) {
    f_min1to8pairs(2,0,0,
    Fn01(x,y,z),  Fn02(x,y,z)  ,0,0,0,0,0,0,
    FnHuge(x,y,z),FnHuge(x,y,z),0,0,0,0,0,0)
}
// Add 10 to third arg. It sets f_min1to8pairs to return map indexes.
#declare Fn03patv = function (x,y,z) {
    f_min1to8pairs(2,0,10,
    Fn01(x,y,z),  Fn02(x,y,z),  0,0,0,0,0,0,
    FnHuge(x,y,z),FnHuge(x,y,z),0,0,0,0,0,0)
}
#declare ChartreuseGreen = srgb <0.50196,1,0>;
#declare Isosurface00 = isosurface {
    function { Fn00(x,y,z) }
    contained_by { box { -1.0,1.0 } }
    report on
    threshold 0
    accuracy 0.0005
    max_gradient 6.8
    max_trace 1
    pigment { ChartreuseGreen }
}
#declare Isosurface01 = isosurface {
    function { Fn03(x,y,z) }
    contained_by { box { -1.0,1.0 } }
    report on
    threshold 0
    accuracy 0.0005
    max_gradient 1.4
    max_trace 2
}
#declare Grey10 = srgb <0.1,0.1,0.1>;
#declare Green = srgb <0,1,0>;
#declare Yellow = srgb <1,1,0>;
#declare Grey20 = srgb <0.2,0.2,0.2>;
#declare ColorMap01 = color_map {
    [ 0/8 Grey10 ]
    [ 1/8 Green ]
    [ 2/8 Yellow ]
    [ 3/8 Grey20 ]
}
// Use the version of Fn03* set up to return color_map index values
#declare Pigment01 = pigment {
    function { Fn03patv(x,y,z) }
    color_map { ColorMap01 }
}
#declare Texture01 = texture { pigment { Pigment01 } }
#declare Object01 = object { Isosurface01 texture { Texture01 } }

//--- scene ---
    camera { Camera00 }
    light_source { Light00 }
//  object { Isosurface00 }
    object { Object01 }
